@@ -20,8 +20,7 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 
-#undef DEBUG_CALLS
-
+#include <IOKit/audio/IOAudioDebug.h>
 #include <IOKit/audio/IOAudioControl.h>
 #include <IOKit/audio/IOAudioControlUserClient.h>
 #include <IOKit/audio/IOAudioTypes.h>
@@ -36,8 +35,8 @@
 OSDefineMetaClassAndStructors(IOAudioControl, IOService)
 OSMetaClassDefineReservedUsed(IOAudioControl, 0);
 OSMetaClassDefineReservedUsed(IOAudioControl, 1);
+OSMetaClassDefineReservedUsed(IOAudioControl, 2);
 
-OSMetaClassDefineReservedUnused(IOAudioControl, 2);
 OSMetaClassDefineReservedUnused(IOAudioControl, 3);
 OSMetaClassDefineReservedUnused(IOAudioControl, 4);
 OSMetaClassDefineReservedUnused(IOAudioControl, 5);
@@ -66,18 +65,65 @@ void IOAudioControl::sendChangeNotification(UInt32 notificationType)
     OSCollectionIterator *iterator;
     IOAudioControlUserClient *client;
     
-    if (!userClients) {
+    if (!userClients || !isStarted) {
         return;
     }
 
-    iterator = OSCollectionIterator::withCollection(userClients);
-    if (iterator) {
-        while (client = (IOAudioControlUserClient *)iterator->getNextObject()) {
-            client->sendChangeNotification(notificationType);
-        }
+	// If we're doing a config change, just queue the notification for later.
+	if (reserved->providerEngine->configurationChangeInProgress) {
+		OSNumber *notificationNumber;
+		UInt32		i, count;
+		bool		dupe = FALSE;
 
-        iterator->release();
-    }
+		if (!reserved->notificationQueue) {
+			reserved->notificationQueue = OSArray::withCapacity (1);
+			if (!reserved->notificationQueue) {
+				return;
+			}
+		}
+
+		notificationNumber = OSNumber::withNumber (notificationType, sizeof (notificationType) * 8);
+		if (!notificationNumber)
+			return;
+
+		// Check to see if this is a unique notification, there is no need to send dupes.
+		count = reserved->notificationQueue->getCount ();
+		for (i = 0; i < count; i++) {
+			if (notificationNumber->isEqualTo ((OSNumber *)reserved->notificationQueue->getObject (i))) {
+				dupe = TRUE;
+				break;		// no need to send duplicate notifications
+			}
+		}
+		if (!dupe) {
+			reserved->notificationQueue->setObject (notificationNumber);
+		}
+		notificationNumber->release ();
+	} else {
+		iterator = OSCollectionIterator::withCollection(userClients);
+		if (iterator) {
+			while (client = (IOAudioControlUserClient *)iterator->getNextObject()) {
+				client->sendChangeNotification(notificationType);
+			}
+	
+			iterator->release();
+		}
+	}
+}
+
+void IOAudioControl::sendQueuedNotifications(void)
+{
+	UInt32				i;
+	UInt32				count;
+
+	// Send our the queued notications and release the queue.
+	if (reserved && reserved->notificationQueue) {
+		count = reserved->notificationQueue->getCount ();
+		for (i = 0; i < count; i++) {
+			sendChangeNotification(((OSNumber *)reserved->notificationQueue->getObject(i))->unsigned32BitValue());
+		}
+		reserved->notificationQueue->release();
+		reserved->notificationQueue = NULL;
+	}
 }
 
 // Original code here...
@@ -146,6 +192,13 @@ bool IOAudioControl::init(UInt32 type,
         return false;
     }
     
+	reserved = (ExpansionData *)IOMalloc (sizeof(struct ExpansionData));
+	if (!reserved) {
+		return false;
+	}
+
+	reserved->providerEngine = NULL;
+	reserved->notificationQueue = NULL;
     isStarted = false;
 
     return true;
@@ -202,9 +255,7 @@ UInt32 IOAudioControl::getUsage()
 
 void IOAudioControl::free()
 {
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::free()\n", this);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::free()", this);
 
     if (userClients) {
         // should we do some sort of notification here?
@@ -231,6 +282,16 @@ void IOAudioControl::free()
         workLoop = NULL;
     }
 
+	if (reserved) {
+		if (reserved->notificationQueue) {
+			reserved->notificationQueue->release();
+			reserved->notificationQueue = NULL;
+		}
+
+		IOFree (reserved, sizeof (struct ExpansionData));
+		reserved = NULL;
+	}
+
     super::free();
 }
 
@@ -241,6 +302,7 @@ bool IOAudioControl::start(IOService *provider)
     }
 
     isStarted = true;
+	reserved->providerEngine = OSDynamicCast (IOAudioEngine, provider);
 
     return true;
 }
@@ -265,9 +327,7 @@ bool IOAudioControl::attachAndStart(IOService *provider)
 
 void IOAudioControl::stop(IOService *provider)
 {
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::stop(%p)\n", this, provider);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::stop(%p)", this, provider);
 
     if (userClients && (userClients->getCount() > 0)) {
         IOCommandGate *cg;
@@ -340,13 +400,11 @@ IOReturn IOAudioControl::setValue(OSObject *newValue)
 {
     IOReturn result = kIOReturnSuccess;
     
-#ifdef DEBUG_CALLS
     if (OSDynamicCast(OSNumber, newValue)) {
-        IOLog("IOAudioControl[%p]::setValue(int = %d)\n", this, ((OSNumber *)newValue)->unsigned32BitValue());
+        audioDebugIOLog(3, "IOAudioControl[%p]::setValue(int = %d)", this, ((OSNumber *)newValue)->unsigned32BitValue());
     } else {
-        IOLog("IOAudioControl[%p]::setValue(%p)\n", this, newValue);
+        audioDebugIOLog(3, "IOAudioControl[%p]::setValue(%p)", this, newValue);
     }
-#endif
 
     if (newValue) {
         if (!value || !value->isEqualTo(newValue)) {
@@ -419,9 +477,7 @@ IOReturn IOAudioControl::hardwareValueChanged(OSObject *newValue)
 {
     IOReturn result = kIOReturnSuccess;
 
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::hardwareValueChanged(%p)\n", this, newValue);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::hardwareValueChanged(%p)", this, newValue);
     
     if (newValue) {
         if (!value || !value->isEqualTo(newValue)) {
@@ -477,9 +533,7 @@ IOReturn IOAudioControl::performValueChange(OSObject *newValue)
 {
     IOReturn result = kIOReturnError;
     
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::performValueChange(%p)\n", this, newValue);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::performValueChange(%p)", this, newValue);
 
     if (valueChangeHandler.intHandler != NULL) {
         switch(valueChangeHandlerType) {
@@ -658,9 +712,7 @@ IOReturn IOAudioControl::newUserClient(task_t task, void *securityID, UInt32 typ
     IOReturn result = kIOReturnSuccess;
     IOAudioControlUserClient *client = NULL;
     
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::newUserClient()\n", this);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::newUserClient()", this);
 
     result = createUserClient(task, securityID, type, &client);
     
@@ -694,9 +746,7 @@ IOReturn IOAudioControl::newUserClient(task_t task, void *securityID, UInt32 typ
 
 void IOAudioControl::clientClosed(IOAudioControlUserClient *client)
 {
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::clientClosed(%p)\n", this, client);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::clientClosed(%p)", this, client);
 
     if (client) {
         IOCommandGate *cg;
@@ -753,9 +803,7 @@ IOReturn IOAudioControl::detachUserClientsAction(OSObject *owner, void *arg1, vo
 
 IOReturn IOAudioControl::addUserClient(IOAudioControlUserClient *newUserClient)
 {
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::addUserClient(%p)\n", this, newUserClient);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::addUserClient(%p)", this, newUserClient);
 
     assert(userClients);
 
@@ -766,9 +814,7 @@ IOReturn IOAudioControl::addUserClient(IOAudioControlUserClient *newUserClient)
 
 IOReturn IOAudioControl::removeUserClient(IOAudioControlUserClient *userClient)
 {
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::removeUserClient(%p)\n", this, userClient);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::removeUserClient(%p)", this, userClient);
 
     assert(userClients);
 
@@ -789,9 +835,7 @@ IOReturn IOAudioControl::detachUserClients()
 {
     IOReturn result = kIOReturnSuccess;
     
-#ifdef DEBUG_CALLS
-    IOLog("IOAudioControl[%p]::detachUserClients()\n", this);
-#endif
+    audioDebugIOLog(3, "IOAudioControl[%p]::detachUserClients()", this);
     
     assert(userClients);
     
