@@ -68,6 +68,9 @@
 
 #define kIOAudioDeviceConfigurationAppKey	"IOAudioDeviceConfigurationApplication"
 
+#define kIOAudioDeviceCanBeDefaults			"IOAudioDeviceCanBeDefaults"
+
+#define kIOAudioDeviceModelIDKey			"IOAudioDeviceModelID"
 
 /*****
  *
@@ -118,6 +121,10 @@
 
 #define kIOAudioEngineDescriptionKey				"IOAudioEngineDescription"
 
+#define kIOAudioEngineClockIsStableKey				"IOAudioEngineClockIsStable"
+
+#define kIOAudioEngineClockDomainKey				"IOAudioEngineClockDomain"
+
 /*!
  * @defined kIOAudioEngineFullChannelNamesKey
  * @abstract The key in the IORegistry for the IOAudioEngine's dictionary of fully constructed names for each channel keyed by the device channel
@@ -369,8 +376,12 @@
 
 #define kIOAudioLevelControlRangesKey		"IOAudioLevelControlRanges"
 
+#define kIOAudioLevelControlUseLinearScale	"IOAudioLevelControlUseLinearScale"
+
 #define kIOAudioSelectorControlAvailableSelectionsKey	"IOAudioSelectorControlAvailableSelections"
 #define kIOAudioSelectorControlSelectionValueKey		"IOAudioSelectorControlSelectionValue"
 #define kIOAudioSelectorControlSelectionDescriptionKey	"IOAudioSelectorControlSelectionDescriptionKey"
 
+#define kIOAudioSelectorControlClockSourceKey				"IOAudioSelectorControlClockSourceKey"
+
 #endif /* _IOAUDIODEFINES_H */
